#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <errno.h>
#include <sys/time.h>
#include <unistd.h>
#include <dbus/dbus.h>
#include <dbus/dbus-glib.h>
#include "bluebus.h"
#include "grays_marshal.h"
#include <gtk/gtk.h>

#define DBUS_SERVICE_BZ   "org.bluez"
#define DBUS_PATH_MGR      "/"
#define DBUS_INTERFACE_MGR "org.bluez.Manager"

static GError *error = NULL;
static void write_error (char *message,
			 GError **error)
{
    if (error == NULL || *error == NULL) {
	return;
    }
    g_printerr ("%s: %s\n", message, (*error)->message);
    g_clear_error (error);
}

static void prop_changed (DBusGProxy *object,
                          const char *prop,
                          GValue *value,
                          void *user_data)
{
    G_rays *g = (G_rays *)user_data; 
    if(0 == strcmp(prop,"Discovering"))
    {
        gboolean disco = g_value_get_boolean (value);
        if(disco)
        {
            g->ndisc++;
            if(g->ndisc == 3)
            {
                g->i_disco = 0;
                dbus_g_proxy_call (g->adapter, "StopDiscovery", NULL,
                                   G_TYPE_INVALID, G_TYPE_INVALID);
                gtk_main_quit ();
            }
        }
    }
}

static void device_found (DBusGProxy *object,
                          const char *address, GHashTable *props,
                          void *user_data)
{
   GValue *value;
   const char *name;
   G_rays *g = (G_rays *)user_data;
   
   value = g_hash_table_lookup (props, "Name");
   name = value ? g_value_get_string (value) : NULL;
   if(strcmp(name, "G-Rays2") == 0)
   {
       char buf[128];
       g->i_disco = 0;
       dbus_g_proxy_call (g->adapter, "StopDiscovery", NULL,
			  G_TYPE_INVALID, G_TYPE_INVALID);
       char *s,*d;
       d = stpcpy(buf, g->path);
       d = stpcpy(d, "/dev_");
       for(s=(char *)address; *s; s++, d++)
       {
	   *d = *s;
	   if(*d == ':')
		   *d = '_';
       }
       *d = 0;
       g->ser = dbus_g_proxy_new_for_name (g->bus, "org.bluez", 
					buf, "org.bluez.Serial");
       if (!dbus_g_proxy_call (g->ser, "Connect", &error,
			       G_TYPE_STRING, "spp",
			       G_TYPE_INVALID,
			       G_TYPE_STRING, &g->rfcom,
			       G_TYPE_INVALID))
       {
	   write_error ("Connect: ", &error);
       }
       else
       {
           gtk_main_quit ();
       }
   }
}

void g_ray_disconnect(G_rays *g)
{
    if(g->ser)
    {
        dbus_g_proxy_call (g->ser, "Disconnect", &error,
                           G_TYPE_STRING, g->rfcom,
                           G_TYPE_INVALID,
                           G_TYPE_INVALID);
        g_object_unref (g->ser);
        g->ser  = NULL;
    }
    
    if(g->rfcom)
    {
        g_free (g->rfcom);
        g->rfcom = NULL;
    }

    
    if(g->adapter)
    {
        if(g->i_disco)
        {
            dbus_g_proxy_call (g->adapter, "StopDiscovery", NULL,
                               G_TYPE_INVALID, G_TYPE_INVALID);
        }
        g_object_unref (g->adapter);
        g->adapter = NULL;
    }

    if(g->path)
    {
        g_free (g->path);
        g->path = NULL;
    }
}

void g_ray_unref(G_rays *g)
{
    if(g)
    {
        g_ray_disconnect(g);
    }
    g_free(g);
}


G_rays * g_ray_new(void)
{
    G_rays *g = g_malloc0(sizeof(G_rays));
    return g;
}

int g_ray_connect(G_rays *g)
{
    int res = -1;
    static DBusGProxy *mgr;    
    g->bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);

    if (g->bus)
    {
	mgr = dbus_g_proxy_new_for_name (g->bus, "org.bluez", 
					 "/", "org.bluez.Manager");
	if(mgr)
        {
            if (!dbus_g_proxy_call (mgr, "DefaultAdapter",
                                    &error,
                                    G_TYPE_INVALID,
                                    DBUS_TYPE_G_OBJECT_PATH,
                                    &g->path, G_TYPE_INVALID))
            {
                write_error ("Default Adapter: ", &error);
            }
        }
        g_object_unref (mgr);
        mgr = NULL;
        if(g->path)
        {
            g->adapter = dbus_g_proxy_new_for_name (g->bus, "org.bluez",
                                             g->path, "org.bluez.Adapter");
            if(g->adapter)
            {
                dbus_g_object_register_marshaller(grays_marshal_VOID__STRING_BOXED,
                                                  G_TYPE_NONE, G_TYPE_STRING,
                                                  G_TYPE_VALUE, G_TYPE_INVALID);
        
                dbus_g_proxy_add_signal (g->adapter, "DeviceFound",
                                         G_TYPE_STRING,
                                         dbus_g_type_get_map
                                         ("GHashTable",
                                          G_TYPE_STRING, G_TYPE_VALUE),
                                         G_TYPE_INVALID);
                dbus_g_proxy_connect_signal (g->adapter, "DeviceFound",
                                             G_CALLBACK (device_found),
                                             g, NULL);
                
                dbus_g_proxy_add_signal (g->adapter, "PropertyChanged",
                                         G_TYPE_STRING, G_TYPE_VALUE,
                                         G_TYPE_INVALID);
                dbus_g_proxy_connect_signal (g->adapter, "PropertyChanged",
                                             G_CALLBACK (prop_changed),
                                             g, NULL);

                g->i_disco = 1;
                dbus_g_proxy_call (g->adapter, "StartDiscovery", &error,
                                   G_TYPE_INVALID, G_TYPE_INVALID);
                if (error != NULL)
                {
                    write_error("Discover: ", &error);
                }
                else
                {
                    res = 0;
                }
            }
        }
    }
    return res;
}

char *g_ray_get_device (G_rays *g)
{
    return g->rfcom;
}

#ifdef MAKE_MAIN
int main(int argc, char **argv)
{
    G_rays *g;

    gtk_init (&argc, &argv);
    g = g_ray_new();
    if(g && g_ray_connect (g) == 0)
    {
        char *gdev;
        gtk_main();
        gdev = g_ray_get_device (g);
        if(gdev)
        {
            printf("Device %s\n", gdev);
            sleep(5);
        }
        g_ray_disconnect (g);
        g_ray_unref(g);
    }
    return 0;
}
#endif

